// SPDX-License-Identifier: MIT
pragma solidity >=0.8.0 <0.9.0;

import "../IGateway.sol";
import "../IApplication.sol";

contract OutBoundCase is IApplication {
    IGateway public gatewayContract;
    string public greeting;

    event RequestFromRouterEvent(string indexed bridgeContract, bytes data);

    constructor(address payable gatewayAddress) {
        gatewayContract = IGateway(gatewayAddress);
    }

    function sendStringPayloadToRouter(
        string memory sampleString,
        string memory routerBridgeContract
    ) external payable {
        bytes memory stringPaylaod = abi.encode(sampleString);
        bytes memory payload = abi.encode(1, stringPaylaod);
        gatewayContract.requestToRouter{ value: msg.value }(0, "", payload, routerBridgeContract, 0, "0x");
    }

    function sendRequestToRouter(
        uint64 chainType,
        string memory chainId,
        address destinationContractAddress,
        string memory str,
        string memory routerBridgeContract
    ) public payable {
        bytes memory innerPayload = abi.encode(chainType, chainId, str, destinationContractAddress);
        bytes memory payload = abi.encode(2, innerPayload);
        gatewayContract.requestToRouter{ value: msg.value }(0, "", payload, routerBridgeContract, 0, "0x");
    }

    function handleRequestFromRouter(string memory sender, bytes memory payload) external {
        // This check is to ensure that the contract is called from the Gateway only.
        require(msg.sender == address(gatewayContract));
        string memory sampleStr = abi.decode(payload, (string));

        require(
            keccak256(abi.encodePacked(sampleStr)) != keccak256(abi.encodePacked("")),
            "please provide non-empty string"
        );
        greeting = sampleStr;
        emit RequestFromRouterEvent(sender, payload);
    }

    receive() external payable {}
}
