// SPDX-License-Identifier: MIT
pragma solidity >=0.8.0 <0.9.0;

import "../IGateway.sol";
import "../IApplication.sol";
import "hardhat/console.sol";

contract ReEntrant is IApplication {
    IGateway public gatewayContract;
    string public greeting;
    uint64 eventIdentifier;

    event RequestFromRouterEvent(string indexed bridgeContract, bytes data);

    constructor(address payable gatewayAddress) {
        gatewayContract = IGateway(gatewayAddress);
    }

    function sendStringPayloadToRouter(
        string memory sampleString,
        string memory routerBridgeContract
    ) external payable {
        console.logString(sampleString);
        bytes memory stringPaylaod = abi.encode(sampleString);
        bytes memory payload = abi.encode(1, stringPaylaod);
        eventIdentifier = gatewayContract.requestToRouter{ value: msg.value }(
            0,
            "",
            payload,
            routerBridgeContract,
            0,
            "0x"
        );
    }

    function sendRequestToRouter(
        uint64 chainType,
        string memory chainId,
        address destinationContractAddress,
        string memory str,
        string memory routerBridgeContract
    ) public payable {
        bytes memory innerPayload = abi.encode(chainType, chainId, str, destinationContractAddress);
        bytes memory payload = abi.encode(2, innerPayload);
        eventIdentifier = gatewayContract.requestToRouter{ value: msg.value }(
            0,
            "",
            payload,
            routerBridgeContract,
            0,
            "0x"
        );
    }

    function handleRequestFromRouter(string memory sender, bytes memory payload) external {
        // This check is to ensure that the contract is called from the Gateway only.
        require(msg.sender == address(gatewayContract));
        string memory sampleStr = abi.decode(payload, (string));

        require(keccak256(abi.encodePacked(sampleStr)) != keccak256(abi.encodePacked("")));
        greeting = sampleStr;

        if (keccak256(abi.encodePacked(sampleStr)) == keccak256(abi.encodePacked("Hello RE-ENTRANCY"))) {
            this.sendStringPayloadToRouter(sampleStr, sender);
        }
        emit RequestFromRouterEvent(sender, payload);
    }

    receive() external payable {}
}
